#include <stdio.h>
#include <stdlib.h>

#include "date.h"


struct date_s {
    int values[3];
};

struct date_s* date_from_string(const char* s)
{
    struct date_s* d = malloc(sizeof(struct date_s));
    sscanf(s, "%d/%d/%d", &d->values[DAY], &d->values[MONTH], &d->values[YEAR]);
    return d;
}

void date_free(struct date_s* date)
{
    free(date);
}

void date_to_string(const struct date_s* const date, char sep, char* str)
{
    sprintf(str, "%02d%c%02d%c%d", date->values[DAY], sep, date->values[MONTH], sep, date->values[YEAR]);
}

int date_compare(const struct date_s* const d1, const struct date_s* const d2)
{
    return (d1->values[YEAR] * 365 + d1->values[MONTH] * 31 + d1->values[DAY]) - (d2->values[YEAR] * 365 + d2->values[MONTH] * 31 + d2->values[DAY]);
}

int date_get_value(const struct date_s* const date, const enum date_value_e value)
{
    switch (value)
    {
        case YEAR:
            return date->values[YEAR];
        case MONTH:
            return date->values[MONTH];
        case DAY:
            return date->values[DAY];
        default:
            return -1;
    }
}

static inline int year_is_bissextile(const int year)
{
    return (year % 4 == 0 && year % 100 != 0) || (year % 400 == 0);
}

int date_is_bissextile(const struct date_s* const date)
{
    return year_is_bissextile(date->values[YEAR]);
}
