#include <math.h>
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char* argv[])
{
    if (argc != 2)
    {
        printf("Usage : %s <fichier>\n", argv[0]);
        return EXIT_FAILURE;
    }

    FILE* fd = fopen(argv[1], "r");
    if (fd == NULL)
    {
        perror("Erreur à l'ouverture du fichier");
        return EXIT_FAILURE;
    }

    int c;

    /* We handle all cases:
     * - empty file
     * - file with only one line not terminating by '\n'
     * - file with several lines, last one does not terminate by '\n'
     * - file terminating with '\n'
     */
    int nb_lines = 0;
    int nb_chars_on_line = 0;
    while ((c = fgetc(fd)) != EOF)
    {
        if (c == '\n')
        {
            nb_lines++;
            nb_chars_on_line = 0;
        }
        else
        {
            nb_chars_on_line++;
        }
    }

    if (nb_chars_on_line)
    {
        nb_lines++;
    }

    char fmt_str[20];
    sprintf(fmt_str, "%%%dd ", (int) ceil(log10(nb_lines)));

    fseek(fd, 0, SEEK_SET);

    int i_line = 1;
    int new_line = 1;
    while ((c = fgetc(fd)) != EOF)
    {
        if (new_line)
        {
            printf(fmt_str, i_line);
            i_line++;
            new_line = 0;
        }
        printf("%c", c);
        if (c == '\n')
        {
            new_line = 1;
        }
    }

    fclose(fd);

    return EXIT_SUCCESS;
}
